/*
	File:		ValidDatePatch.c

	Contains:	Code.
	
				In MacOS 8.5 the 0x8204ffe4 selector for ScriptUtils went away.
				This is bad because it was a compatibility selector for ValidDate.
				The correct selectory for ValidDate is in fact 0x820Cffe4, but the
				incorrect one was used by Think Pascal's interfaces.  Despite the
				fact that the selector was corrected in the interfaces shipped
				with ETO6 in 1991, some older apps used the old (and incorrect)
				selector, and this was supported in releases of MacOS until 8.5.
				
				It looks like this was removed in 8.5, so this patch coerces the
				selector into the correcrt selector.  Bear in mind that this method
				is ineffecient, since it will cause a mode switch to 68k code each
				and every time the ScriptUtil trap is called.  This will cause a 
				performance hit.
				
				The (incorrect) selector will hopefully re-appear for 8.6.  For
				compatibility reasons.

	Version:	1.0

	Copyright:	 1999 by Apple Computer, Inc., all rights reserved.

	File Ownership:

		DRI:				?????

		Other Contact:		?????

		Technology:			Making the system more stable with patches

	Writers:


	Change History (most recent first):

*/

//*****************************************************************************

// Public headers
#include <MacTypes.h>
#include <Traps.h>
#include <Resources.h>
#include <Gestalt.h>

// Metrowerks specific headers
#include <A4Stuff.h>
#include <SetUpA4.h>

//*****************************************************************************

//
// Constants
//

enum
{
	// type and ID of this INIT resource
	kInitType				= 'INIT',
	kInitID					= 0,
	
	// Only load if Mac OS 8.5.x
	kSystemVersion	= 0x00000850,
	
	kBadSelector	= 0x8204ffe4,
	kGoodSelector	= 0x820Cffe4
};

//*****************************************************************************

//
// Prototypes
//

// prototypes for assembly functions
static asm void AsmCode(void);
static asm void ScriptUtilPatch(void);
static void InstallScriptUtilPatch(void);

//*****************************************************************************

static asm void AsmCode(void)
{

//*****************************************************************************

//
//	ScriptUtilPatch is our patch on _ScriptUtil that fixes the bad selector
//
entry	ScriptUtilPatch
ScriptUtilPatch:
		cmp.l	#kBadSelector,4(sp)
		bne.s	JumpOldScriptUtil
		
		move.l	#kGoodSelector,4(sp)

JumpOldScriptUtil:		
		move.l	OldScriptUtil,-(sp)				// get the old routine
		rts										// jump to old routine

OldScriptUtil:
		ds.l	1								// storage for old ScriptUtil code

//*****************************************************************************

//
//	InstallScriptUtilPatch installs our patch on _ScriptUtil.
//
entry InstallScriptUtilPatch
InstallScriptUtilPatch:

		move.l	#_ScriptUtil,d0
		_GetToolTrapAddress						// a0 = old ScriptUtil
		
		lea		OldScriptUtil,a1
		move.l	a0,(a1)							// Save old ScriptUtil
		
		lea		ScriptUtilPatch,a0					// Install the ScriptUtil patch
		move.l	#_ScriptUtil,d0
		_SetToolTrapAddress
		
		rts
}

//******************************************************************************

//
//	main
//
//	Set up for the patch and then install it.
//
void main(void)
{
	EnterCodeResource();
	
	Handle			initHandle;
	long			response;
	
	PrepareCallback();

	if ( (Gestalt(gestaltSystemVersion, &response) == noErr) &&
		 ((response & 0x00000FF0) == kSystemVersion) )
	{
		// Detach ourselves
		initHandle = GetResource(kInitType, kInitID);
		if ( initHandle != NULL )
		{
			DetachResource(initHandle);
			if ( ResError() == noErr )
			{
				// Install patches
				InstallScriptUtilPatch();
			}
			else
			{
				goto Cleanup;
			}
		}
		else
		{
			goto Cleanup;
		}
		goto Exit;
	}
	
Cleanup:

Exit:
	ExitCodeResource();
}

//*****************************************************************************

